import 'package:easeim_flutter_demo/widgets/common_widgets.dart';
import 'package:flutter/material.dart';
import 'package:flutter_smart_dialog/flutter_smart_dialog.dart';
import 'package:im_flutter_sdk/im_flutter_sdk.dart';

class RegisterPage extends StatefulWidget {
  @override
  State<StatefulWidget> createState() => _RegisterPageState();
}

class _RegisterPageState extends State<RegisterPage> {
  late TextEditingController _usernameController;
  late TextEditingController _pwdController;
  late TextEditingController _confPwdController;
  bool _agreeProtocal = false;

  @override
  void initState() {
    super.initState();
    updateTextFieldController();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      resizeToAvoidBottomInset: false,
      body: ConstrainedBox(
        constraints: BoxConstraints.expand(),
        child: GestureDetector(
          onTap: () => hiddenKeyboard(),
          child: Container(
            padding: EdgeInsets.zero,
            decoration: BoxDecoration(
              image: DecorationImage(
                colorFilter: ColorFilter.mode(
                  Color.fromRGBO(0, 0, 0, 0.3),
                  BlendMode.srcOver,
                ),
                alignment: Alignment.bottomCenter,
                image: AssetImage('images/login_bg.jpg'),
                fit: BoxFit.cover,
              ),
            ),
            child: SafeArea(
              child: Stack(
                alignment: Alignment.topCenter,
                children: [
                  Positioned(
                    left: 24,
                    top: 10,
                    child: SizedBox(
                      width: 24,
                      height: 24,
                      child: IconButton(
                        padding: EdgeInsets.zero,
                        icon: Image.asset(
                          'images/register_back.png',
                          fit: BoxFit.fill,
                        ),
                        onPressed: () => Navigator.pop(context),
                      ),
                    ),
                  ),
                  Positioned(
                    child: Column(
                      mainAxisSize: MainAxisSize.min,
                      crossAxisAlignment: CrossAxisAlignment.center,
                      children: [
                        SizedBox(
                          height: 88,
                        ),
                        Text(
                          '注册账号',
                          style: TextStyle(
                            color: Colors.white,
                            fontSize: 18,
                          ),
                        ),
                        // 用户名
                        Container(
                          padding: EdgeInsets.fromLTRB(33, 38, 33, 0),
                          child: loginRegisterTextField(
                            hintText: '用户名',
                            controller: _usernameController,
                            rightIcon: IconButton(
                              padding: EdgeInsets.zero,
                              icon: Image.asset(
                                'images/login_username_clear.png',
                              ),
                              onPressed: () {
                                _usernameController.text = '';
                              },
                            ),
                          ),
                        ),
                        // 密码
                        Container(
                          padding: EdgeInsets.fromLTRB(33, 20, 33, 0),
                          child: loginRegisterTextField(
                            hintText: '密码',
                            isPwd: true,
                            controller: _pwdController,
                          ),
                        ),
                        // 确认密码
                        Container(
                          padding: EdgeInsets.fromLTRB(33, 20, 33, 0),
                          child: loginRegisterTextField(
                            hintText: '确认密码',
                            isPwd: true,
                            controller: _confPwdController,
                          ),
                        ),
                        Container(
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.start,
                            crossAxisAlignment: CrossAxisAlignment.center,
                            children: [
                              Spacer(),
                              Container(
                                width: 30,
                                height: 30,
                                child: IconButton(
                                  padding: EdgeInsets.zero,
                                  onPressed: () {
                                    setState(() {
                                      _agreeProtocal = !_agreeProtocal;
                                    });
                                  },
                                  icon: Image.asset(
                                    _agreeProtocal
                                        ? 'images/login_protocal_selected.png'
                                        : 'images/login_protocal_unselected.png',
                                    fit: BoxFit.cover,
                                  ),
                                ),
                              ),
                              TextButton(
                                child: Text(
                                  '同意 服务条款 与 隐私协议',
                                  style: TextStyle(
                                    color: Colors.white,
                                    fontWeight: FontWeight.w300,
                                  ),
                                ),
                                onPressed: () {},
                                // style: ButtonStyle(
                                //     padding: MaterialStateProperty.all(
                                //         EdgeInsets.only(bottom: 20))),
                              ),
                              Spacer(),
                            ],
                          ),
                        ),
                        SizedBox(
                          height: 30,
                        ),
                        loginRegisterButton(
                          enable: _agreeProtocal,
                          margin: EdgeInsets.only(left: 33, right: 33),
                          onPressed: _doRegisterAction,
                          title: '注册',
                          beginColor: Colors.blue[400]!,
                          endColor: Colors.blue[800]!,
                          disableColor: Colors.black38,
                        ),
                      ],
                    ),
                  ),
                ],
              ),
            ),
          ),
          behavior: HitTestBehavior.translucent,
        ),
      ),
    );
  }

  void updateTextFieldController() {
    _usernameController = TextEditingController();
    _usernameController.addListener(() {});
    _pwdController = TextEditingController();
    _pwdController.addListener(() {});
    _confPwdController = TextEditingController();
    _confPwdController.addListener(() {});
  }

  hiddenKeyboard() {
    FocusScope.of(context).requestFocus(FocusNode());
  }

  _doRegisterAction() async {
    try {
      SmartDialog.showLoading(msg: '注册中...');
      await EMClient.getInstance
          .createAccount(_usernameController.text, _confPwdController.text);

      SmartDialog.showToast('注册成功');
      Navigator.of(context).pushReplacementNamed('/login');
    } on EMError catch (e) {
      SmartDialog.showToast('注册失败 $e');
    } finally {
      SmartDialog.dismiss();
    }
  }
}
